'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (options = {}) {
  const userConfig = (0, _getConfig2.default)(options.config);

  options = (0, _lodash2.default)({
    input: './src/index.js',
    format: ['cjs'],
    outDir: './dist',
    filename: 'index'
  }, userConfig, options);

  // for backward-compat
  if (options.entry) {
    options.input = options.entry;
  }

  let formats = options.format;

  if (typeof formats === 'string') {
    if (formats === 'all') {
      formats = ['cjs', 'umd', 'es'];
    } else if (formats.indexOf(',') > -1) {
      formats = formats.split(',').map(v => v.trim());
    } else {
      formats = [formats];
    }
  } else if (!Array.isArray(formats)) {
    throw new TypeError('Expect "format" to be a string or Array');
  }

  return Promise.all(formats.map(format => {
    const rollupOptions = (0, _getRollupOptions2.default)(options, format);
    if (options.watch) {
      let init;
      return new Promise(resolve => {
        const watcher = rollup.watch(rollupOptions);
        watcher.on('event', event => {
          (0, _switchy2.default)({
            START() {
              (0, _log2.default)(format, 'starting', _chalk2.default.blue);
              if (!init) {
                init = true;
                return resolve();
              }
            },
            BUNDLE_START() {},
            BUNDLE_END() {
              (0, _log2.default)(format, 'bundled', _chalk2.default.green);
            },
            END() {},
            ERROR() {
              (0, _utils.handleRollupError)(event.error);
            },
            FATAL() {
              (0, _utils.handleRollupError)(event.error);
            },
            default() {
              console.error('unknown event', event);
            }
          })(event.code);
        });
      });
    }
    return rollup.rollup(rollupOptions).then(bundle => {
      if (options.write === false) return bundle.generate(rollupOptions.output);
      return bundle.write(rollupOptions.output);
    });
  })).then(result => {
    return result.reduce((res, next, i) => {
      res[(0, _camelcase2.default)(formats[i])] = next;
      return res;
    }, {});
  });
};

var _rollup = require('rollup');

var rollup = _interopRequireWildcard(_rollup);

var _switchy = require('switchy');

var _switchy2 = _interopRequireDefault(_switchy);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _lodash = require('lodash.merge');

var _lodash2 = _interopRequireDefault(_lodash);

var _camelcase = require('camelcase');

var _camelcase2 = _interopRequireDefault(_camelcase);

var _getRollupOptions = require('./get-rollup-options');

var _getRollupOptions2 = _interopRequireDefault(_getRollupOptions);

var _getConfig = require('./get-config');

var _getConfig2 = _interopRequireDefault(_getConfig);

var _utils = require('./utils');

var _log = require('./log');

var _log2 = _interopRequireDefault(_log);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }