'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /* eslint-disable complexity */


exports.default = function (options, format) {
  let compress = false;
  if (format.endsWith('-min')) {
    format = format.replace(/-min$/, '');
    compress = true;
  }

  let plugins = [require('rollup-plugin-json')(options.json)];

  const js = options.js === false ? null : options.js || 'buble';

  let jsOptions = js && options[js] || {};

  // Add default options for buble plugin
  if (js === 'buble') {
    const transforms = jsOptions.transforms;
    jsOptions = _extends({
      objectAssign: 'Object.assign',
      include: ['**/*.{js,jsx,es6,vue}']
    }, jsOptions, {
      transforms: _extends({
        generator: false,
        dangerousForOf: true,
        dangerousTaggedTemplateString: true
      }, transforms)

      // optionally compile async/await to generator function
    });if (js === 'buble' && jsOptions.async !== false) {
      plugins.push(require('rollup-plugin-async')());
    }
  }

  if (options.flow) {
    plugins.push(require('rollup-plugin-flow')());
  }

  if (typeof options.plugins === 'object' || typeof options.plugins === 'string') {
    const _plugins = Array.isArray(options.plugins) ? options.plugins : [options.plugins];
    const extraPlugins = _plugins.map(p => {
      if (typeof p === 'string') {
        return (0, _reqCwd2.default)(`rollup-plugin-${p}`)(options[p]);
      }
      return p;
    });
    plugins = [...plugins, ...extraPlugins];
  }

  if (js) {
    let jsPlugin;
    try {
      jsPlugin = js === 'buble' ? require('rollup-plugin-buble') : (0, _reqCwd2.default)(`rollup-plugin-${js}`);
      plugins.push(jsPlugin(jsOptions));
    } catch (err) {
      if (/missing path/.test(err.message)) {
        throw new Error(`rollup-plugin-${js} was not found in current working directory!`);
      } else {
        throw err;
      }
    }
  }

  if (options.alias) {
    plugins.push(require('rollup-plugin-alias')(options.alias));
  }

  if (options.replace) {
    plugins.push(require('rollup-plugin-replace')(options.replace));
  }

  // env is automatically stringified
  if (options.env) {
    const env = Object.keys(options.env).reduce((res, key) => {
      res[`process.env.${key}`] = JSON.stringify(options.env[key]);
      return res;
    }, {});
    plugins.push(require('rollup-plugin-replace')({
      values: env
    }));
  }

  if (format === 'umd' || options.resolve) {
    const esModules = options.esModules === undefined ? true : options.esModules;
    plugins.push(require('rollup-plugin-node-resolve')(_extends({
      skip: options.skip,
      jsnext: esModules,
      module: esModules,
      browser: options.browser
    }, options.nodeResolve)), require('rollup-plugin-commonjs')(options.commonjs));
  }

  let banner;
  if (options.banner) {
    if (typeof options.banner === 'string') {
      banner = options.banner;
    } else {
      const pkg = typeof options.banner === 'object' ? _extends({}, options.pkg, options.banner) : options.pkg;

      const name = pkg.name;

      if (typeof name !== 'string') {
        throw new TypeError(`Expect "name" in package.json to be a string but got ${typeof name}.`);
      }

      const version = pkg.version ? `v${pkg.version}` : '';
      const year = pkg.year || new Date().getFullYear();

      let author = typeof pkg.author === 'string' ? pkg.author : typeof pkg.author === 'object' ? (0, _stringifyAuthor2.default)(pkg.author) : '';
      author = author ? author : '';

      const license = pkg.license || '';

      banner = '/*!\n' + ` * ${name} ${version}\n` + ` * (c) ${year}-present ${author}\n` + (license && ` * Released under the ${license} License.\n`) + ' */';
    }
  }

  if (compress) {
    plugins.push(require('rollup-plugin-uglify')({
      output: {
        // Preserve banner
        preamble: banner
      }
    }));
  }

  let moduleName = 'index';
  if (options.moduleName) {
    moduleName = options.moduleName;
  } else if (options.filename) {
    moduleName = (0, _camelcase2.default)(options.filename);
  }

  let external;
  if (format === 'cjs') {
    // Exclude .json files in commonjs bundle
    external = id => /\.json$/.test(id);
  }
  external = options.external || external;
  if (external && !Array.isArray(external)) {
    external = [external];
  }

  let globals;
  if (options.globals) {
    globals = options.globals;
    if (typeof globals === 'object') {
      external = [...(external || []), ...Object.keys(globals)];
    }
  }

  if (typeof options.plugins === 'function') {
    plugins = options.plugins(plugins);
  }

  return {
    input: options.input,
    plugins,
    external,
    output: {
      file: getDest(options, format, compress),
      exports: options.exports,
      globals,
      name: moduleName,
      paths: options.paths,
      sourcemap: getMap(options, compress),
      format,
      banner
    }
  };
};

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _camelcase = require('camelcase');

var _camelcase2 = _interopRequireDefault(_camelcase);

var _reqCwd = require('req-cwd');

var _reqCwd2 = _interopRequireDefault(_reqCwd);

var _stringifyAuthor = require('stringify-author');

var _stringifyAuthor2 = _interopRequireDefault(_stringifyAuthor);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getDest(options, format, compress) {
  const filename = options.filename;
  const dir = options.outDir;
  let suffix = '';

  if (format === 'cjs') {
    suffix += '.common';
  } else if (format === 'es') {
    suffix += '.es';
  } else if (format === 'iife') {
    suffix += '.iife';
  }

  if (compress) {
    suffix += '.min.js';
  } else {
    suffix += '.js';
  }

  const output = _path2.default.join(dir, filename + suffix);
  return output;
}

function getMap(options, compress) {
  if (typeof options.map === 'boolean') {
    return options.map;
  }
  return compress;
}